﻿using System;
using System.Text.RegularExpressions;
using System.Globalization;

namespace Demo7
{
    /// @author  Vesa Lappalainen, Antti-Jussi Lakanen
    /// @version 2025
    /// <summary>
    /// Aliohjelmia taulukoille.
    /// Matriisien summa ja liukuluvun erotus merkkijonosta.
    /// (Tämä tiedosto on peräisin demo 6:sta.)
    /// </summary>
    public class Taulukot
    {
        /// <summary>
        /// Lasketaan yhteen kaksi matriisia vastinalkioittain.
        /// Mikäli matriisit eivät ole yhtäsuuria, tehdään tulosmatriisin
        /// kooksi matriisien yhteinen pienin koko.
        /// </summary>
        /// <param name="mat1">ensimmäinen summattava</param>
        /// <param name="mat2">toinen summattava</param>
        /// <returns>taulukkojen summataulukko</returns>
        /// <example>
        /// <pre name="test">
        /// double[,] mat1 = { { 1, 2, 3 }, { 2, 2, 2 }, { 4, 2, 3 } };
        /// double[,] mat2 = { { 9, 2, 8 }, { 1, 2, 5 }, { 3, 19, -3 } };
        /// double[,] mat23 = { { 1, 2, 3 }, { 2, 2, 2 } };
        /// double[,] mat32 = { { 9, 2 }, { 1, 2 }, { 3, 19 } };
        /// double[,] mat = Taulukot.Summa(mat1, mat2);
        /// Demo6.Matriisit.Jonoksi(mat," ","{0}",",") === "10 4 11,3 4 7,7 21 0";
        /// mat = Taulukot.Summa(mat23, mat32);
        /// Demo6.Matriisit.Jonoksi(mat," ","{0}",",") === "10 4,3 4";
        /// double[,] mat41 = { { 1 }, { 2 }, {3} };
        /// double[,] mat42 = { { 9, 2, 8 }, { 1, 2, 5 }, { 3, 19, -3 } };
        /// mat = Taulukot.Summa(mat41, mat42);
        /// Demo6.Matriisit.Jonoksi(mat," ","{0}",",") === "10,3,6";
        /// </pre>
        /// </example>
        public static double[,] Summa(double[,] mat1, double[,] mat2)
        {
            int ny = Math.Min(mat1.GetLength(0), mat2.GetLength(0));
            int nx = Math.Min(mat1.GetLength(1), mat2.GetLength(1));
            double[,] tulos = new double[ny, nx];
            for (int iy = 0; iy < ny; iy++)
                for (int ix = 0; ix < nx; ix++)
                    tulos[iy, ix] = mat1[iy, ix] + mat2[iy, ix];
            return tulos;
        }


        /// <summary>
        /// Erotetaan merkkijonossa olevat luvut reaalilukutaulukoksi.
        /// Luvut voivat olla erotettuna toisistaan välilyönnein,
        /// pilkuin, puolipistein yms. Luvut joita ei voida tulkista,
        /// palautetaan 0:na.
        /// Tyhjät alkiot poistetaan (esim. kaksi erotinmerkkiä peräkkäin).
        /// </summary>
        /// <param name="jono">jono josta lukuja otetaan</param>
        /// <returns>taulukollinen lukuja</returns>
        /// <example>
        /// <pre name="test">
        ///   double[] luvut = Taulukot.ErotaLuvut("2 3 4 5 k      9 ;5");
        ///   String.Join(" ",luvut) === "2 3 4 5 0 9 5";  
        /// </pre>
        /// </example>
        public static double[] ErotaLuvut(string jono)
        {
            string[] osat = jono.Split(new char[] { ' ', ',', ';', '/', ':' }, StringSplitOptions.RemoveEmptyEntries);
            double[] tulos = new double[osat.Length];
            for (int i = 0; i < osat.Length; i++)
            {
                tulos[i] = ErotaDouble(osat[i], 0.0);
            }
            return tulos;
        }

        /// <summary>
        /// Otetaan jonossa oleva reaaliluku.  Jos luku ei ole
        /// mielekäs luku, niin palautetaan oletus. Desimaalimerkkinä 
        /// on aina piste ".".
        /// </summary>
        /// <param name="jono">Jono josta reaaliluku otetaan</param>
        /// <param name="oletus">Mikä arvo annetaan jos jonosta ei saada lukua</param>
        /// <returns>Otettu reaaliluku</returns>
        /// <example>
        /// <pre name="test">
        ///   Taulukot.ErotaDouble("") ~~~ 0.0;
        ///   Taulukot.ErotaDouble(" 2.3 ") ~~~ 2.3;
        ///   Taulukot.ErotaDouble("5 3") ~~~ 5;
        ///   Taulukot.ErotaDouble("5k3") ~~~ 5;
        ///   Taulukot.ErotaDouble("5e3") ~~~ 5000;
        ///   Taulukot.ErotaDouble("5E-3") ~~~ 0.005;
        ///   Taulukot.ErotaDouble("k") ~~~ 0.0;
        ///   Taulukot.ErotaDouble("k",1.0) ~~~ 1.0;
        ///   Taulukot.ErotaDouble("2..3") ~~~ 0.0;
        /// </pre>
        /// </example>
        public static double ErotaDouble(string jono, double oletus = 0.0)
        {
            string reg = @"^([-0-9\.eE]+)(.*)$";
            Match m = Regex.Match(jono.Trim(), reg);
            string tjono = m.Groups[1].Value;
            double tulos = oletus;
            NumberFormatInfo nfi = new NumberFormatInfo
            {
                NumberDecimalSeparator = "."
            };
            if (double.TryParse(tjono, NumberStyles.Any, nfi, out tulos)) return tulos;
            return oletus;
        }


        /// <summary>
        /// Otetaan jonossa oleva reaaliluku.  Jos luku ei ole
        /// mielekäs luku, niin palautetaan oletus.
        /// </summary>
        /// <param name="jono">jono josta reaaliluku otetaan</param>
        /// <param name="oletus">mikä arvo annetaan jos jonosta ei saada lukua</param>
        /// <returns>otettu reaaliluku</returns>
        /// <example>
        /// <pre name="test">
        ///   Taulukot.Erota("",0.0) ~~~ 0.0;
        ///   Taulukot.Erota(" 2.3 ",0.0) ~~~ 2.3;
        ///   Taulukot.Erota("5 3",0.0) ~~~ 5;
        ///   Taulukot.Erota("k",0.0) ~~~ 0.0;
        ///   Taulukot.Erota("k",1.0) ~~~ 1.0;
        ///   Taulukot.Erota("2..3",0.0) ~~~ 0.0;
        /// </pre>
        /// </example>
        public static double Erota(string jono, double oletus)
        {
            return ErotaDouble(jono, oletus);
        }
    }
}
